local MapData = nil

ARH_GREEN = 1
ARH_YELLOW = 2
ARH_RED = 3

CONYARDS = {[ARH_GREEN] = 40, [ARH_YELLOW] = 80, [ARH_RED] = 640}

local minimap_size = {
	indoor = {
		[0] = 300, -- scale
		[1] = 240, -- 1.25
		[2] = 180, -- 5/3
		[3] = 120, -- 2.5
		[4] = 80,  -- 3.75
		[5] = 50,  -- 6
	},
	outdoor = {
		[0] = 466 + 2/3, -- scale
		[1] = 400,       -- 7/6
		[2] = 333 + 1/3, -- 1.4
		[3] = 266 + 2/6, -- 1.75
		[4] = 200,       -- 7/3
		[5] = 133 + 1/3, -- 3.5
	},
}
local minimap_scale = {
	indoor = {
		[0] = 1,
		[1] = 1.25,
		[2] = 5/3,
		[3] = 2.5,
		[4] = 3.75,
		[5] = 6,
	},
	outdoor = {
		[0] = 1,
		[1] = 7/6,
		[2] = 1.4,
		[3] = 1.75,
		[4] = 7/3,
		[5] = 3.5,
	},
}

local function RotateTexture(texture, angle)
	local cos, sin = math.cos(angle), math.sin(angle)
	local p, m = (sin+cos)/2, (sin-cos)/2
	local pp, pm, mp, mm = 0.5+p, 0.5+m, 0.5-p, 0.5-m
	texture:SetTexCoord(pm, mp, mp, mm, pp, pm, mm, pp)
end

local function CreateConTexture(parent, color)
	local t = parent:CreateTexture()
	t:SetBlendMode("ADD")
	t:SetPoint("CENTER", parent, "CENTER", 0, 0)
	t:SetTexture("Interface\\AddOns\\Arh\\img\\con1024_"..color)
	t:Show()

	return t
end

local function CreateLineTexture(parent, contexture, color)
	local t = parent:CreateTexture()
	t:SetBlendMode("ADD")
	t:SetPoint("CENTER", contexture, "CENTER", 0, 0)
	t:SetTexture("Interface\\AddOns\\Arh\\img\\line1024_"..color)
	t:Show()

	return t
end

local function SetTextureColor(texture, color, usealpha)
	local r,g,b,a
	if color == ARH_RED then
		r,g,b,a = 1,0,0,0.1
	elseif color == ARH_YELLOW then
		r,g,b,a = 0.5,0.5,0,0.1
	elseif color == ARH_GREEN then
		r,g,b,a = 0,1,0,0.1
	end

	if usealpha then
		texture:SetVertexColor(r,g,b,a)
	else
		if color==ARH_RED then
			texture:SetVertexColor(r,g,b,0.02) -- red line does not deserve
		else
			texture:SetVertexColor(r,g,b,0.2)
		end
	end
end

function PixelsInYardOnHud_Calc()
	local mapSizePix = Arh_HudFrame:GetHeight()

	local zoom = Minimap:GetZoom()
	local indoors = GetCVar("minimapZoom")+0 == Minimap:GetZoom() and "outdoor" or "indoor"
	local mapSizeYards = minimap_size[indoors][zoom]

	return mapSizePix/mapSizeYards
end
local PixelsInYardOnHud = -1


function UpdateTextureSize(texture, color)
	texture:SetSize(PixelsInYardOnHud * CONYARDS[color]*2, PixelsInYardOnHud * CONYARDS[color]*2)
end

function CreateCon(parent, color)
	local t = CreateConTexture(parent, color)
	SetTextureColor(t, color, true)
	UpdateTextureSize(t, color)

	return t
end
function CreateLine(parent, color, contexture)
	local t = CreateLineTexture(parent, contexture, color)
	SetTextureColor(t, color, false)
	UpdateTextureSize(t, color)

	return t
end


function UpdateConAndLine(texture_con, texture_line, color)
	UpdateTextureSize(texture_con, color)
	texture_con:Show()

	UpdateTextureSize(texture_line, color)
	texture_line:Show()
end

ConsCache = {[ARH_GREEN] = {["items"]={}, ["size"]=0}, [ARH_YELLOW] = {["items"]={}, ["size"]=0}, [ARH_RED] = {["items"]={}, ["size"]=0} }
ConsArray = {}
local ConsArraySize = 0
function GetCached(color)
	if ConsCache[color].size > 0 then
		ConsCache[color].size = ConsCache[color].size - 1
		return ConsCache[color].items[ConsCache[color].size]
	else
		return nil
	end
end
function ReturnAllToCache()
	for i=0,ConsArraySize-1 do
		ConsArray[i].texture:Hide()
		ConsArray[i].texture_line:Hide()
		local c = ConsArray[i].color
		local cs = ConsCache[c].size
		ConsCache[c].items[cs] = {["texture"]=ConsArray[i].texture, ["texture_line"]=ConsArray[i].texture_line}
		ConsCache[c].size = cs+1
	end
	ConsArraySize = 0
end


function AddCon(color, x, y, a)
	local item = GetCached(color)
	if item then
		ConsArray[ConsArraySize] = {["color"]=color, ["texture"]=item.texture, ["texture_line"]=item.texture_line, ["x"]=x, ["y"]=y, ["a"]=a}
		UpdateConAndLine(ConsArray[ConsArraySize].texture, ConsArray[ConsArraySize].texture_line, color)
		
	else
		ConsArray[ConsArraySize] = {["color"]=color, ["texture"]=CreateCon(Arh_HudFrame, color), ["texture_line"]=nil, ["x"]=x, ["y"]=y, ["a"]=a}
		ConsArray[ConsArraySize].texture_line = CreateLine(Arh_HudFrame, color, ConsArray[ConsArraySize].texture)
	end
	ConsArraySize = ConsArraySize+1
end

function UpdateConsSizes()
	local piy = PixelsInYardOnHud_Calc()
	if piy == PixelsInYardOnHud then return end
	PixelsInYardOnHud = piy
	for i=0,ConsArraySize-1 do
		UpdateTextureSize(ConsArray[i].texture, ConsArray[i].color)
		UpdateTextureSize(ConsArray[i].texture_line, ConsArray[i].color)
	end
end

function PlaceCon(id, player_x, player_y, player_a)
	local dx, dy = ConsArray[id].x-player_x, ConsArray[id].y-player_y
	local cos, sin = math.cos(player_a), math.sin(player_a)
	local x = dx*cos - dy*sin
	local y = dx*sin + dy*cos

	ConsArray[id].texture:ClearAllPoints()
	ConsArray[id].texture:SetPoint("CENTER", Arh_HudFrame, "CENTER", x*PixelsInYardOnHud, -y*PixelsInYardOnHud)
	RotateTexture(ConsArray[id].texture, ConsArray[id].a-player_a)

	RotateTexture(ConsArray[id].texture_line, ConsArray[id].a-player_a)
end

function UpdateConsPositions(player_x, player_y, player_a)
	for i=0,ConsArraySize-1 do
		PlaceCon(i, player_x, player_y, player_a)
	end
end

function UpdateCons(player_x, player_y, player_a)
	UpdateConsSizes() -- if minimap zoomed
	UpdateConsPositions(player_x, player_y, player_a)
end

function Arh_ToYards(x, y)
	local level = GetCurrentMapDungeonLevel();
	local map = GetCurrentMapAreaID();
	return MapData:PointToYards(map, level, x, y);	
end

local function Distance(xa, ya, xb, yb)
	return math.sqrt(math.pow(xa-xb,2)+math.pow(ya-yb,2))
end

local function CalcAngle(xa, ya, xb, yb)
	if ya == yb then
		if xa == xb then
			return 0;
		elseif xa > xb then
			return math.pi/2;
		else
			return 3*math.pi/2;
		end
	end
	local t = (xb-xa)/(yb-ya);
	local a = math.atan(t);
	if ya > yb then
		if xa == xb then
			return 0;
		elseif xa > xb then
			return a;
		else
			return a+2*math.pi;
		end
	else
		if xa == xb then
			return math.pi;
		elseif xa > xb then
			return a+math.pi;
		else
			return a+math.pi;
		end
	end
end

local function AddPoint(color)
	SetMapToCurrentZone();
	local x, y = GetPlayerMapPosition("player")
        local jax, jay = Arh_ToYards(x, y)
	a = GetPlayerFacing()

	AddCon(color, jax, jay, a)
end

function Arh_MainFrame_ButtonRed_OnClick()
	AddPoint(ARH_RED);
end

function Arh_MainFrame_ButtonYellow_OnClick()
	AddPoint(ARH_YELLOW);
end

function Arh_MainFrame_ButtonGreen_OnClick()
	AddPoint(ARH_GREEN);
end

function OnGathering()
	ReturnAllToCache()
end

function Arh_MainFrame_ButtonDig_OnClick()
	if Arh_HudFrame:IsVisible() then
		Arh_HudFrame:Hide()
	else
		Arh_HudFrame:Show()
	end
end


local function handler(msg, editbox)
	if Arh_MainFrame:IsVisible() then
		Arh_MainFrame:Hide()
	else
		Arh_MainFrame:Show()
	end
end
SlashCmdList["ARH"] = handler;
SLASH_ARH1 = "/arh"

function Arh_SpellSent(unit,spellcast,rank,target)
	if unit ~= "player" then return end
	if spellcast==GetSpellInfo(73979) then -- "Searching for Artifacts"
		OnGathering();
	end
end

function Arh_MainFrame_OnEvent(self, event, ...)
	if event == "UNIT_SPELLCAST_SENT" then
		Arh_SpellSent(...)
	end
end

function Arh_MainFrame_OnLoad()
	MapData = LibStub("LibMapData-1.0")
	Arh_MainFrame:Show()
	Arh_MainFrame:RegisterEvent("UNIT_SPELLCAST_SENT")

	if ( not BattlefieldMinimap ) then
		BattlefieldMinimap_LoadUI();
	end
	Arh_ArchaeologyDigSites_BattlefieldMinimap:SetParent(BattlefieldMinimap)
	Arh_ArchaeologyDigSites_BattlefieldMinimap:ClearAllPoints()
	Arh_ArchaeologyDigSites_BattlefieldMinimap:SetPoint("TOPLEFT", BattlefieldMinimap)
	Arh_ArchaeologyDigSites_BattlefieldMinimap:SetPoint("BOTTOMRIGHT", BattlefieldMinimap)

	Arh_ArchaeologyDigSites_Minimap:ClearAllPoints()
	Arh_ArchaeologyDigSites_Minimap:SetParent(MinimapCluster)
	Arh_ArchaeologyDigSites_Minimap:EnableMouse(false)
	Arh_ArchaeologyDigSites_Minimap:SetFrameStrata("BACKGROUND")
end

function Arh_MainFrame_OnMouseDown()
	Arh_MainFrame:StartMoving()
end

function Arh_MainFrame_OnMouseUp()
	Arh_MainFrame:StopMovingOrSizing()
end

local function RePositionDigSites_Minimap(self)
	local x, y = GetPlayerMapPosition("player");
	local dx=(x-0.5)*self:GetWidth();
	local dy=(y-0.5)*self:GetHeight();
	self:ClearAllPoints();
	self:SetPoint("CENTER", MinimapCluster, "CENTER", -dx, dy);
end
local old_pw, old_ph = -1, -1
local function UpdateDigSitesSize_Minimap(self)
	local mapWidth = self:GetParent():GetWidth()
	local mapHeight = self:GetParent():GetHeight()
	local mapSizePix = math.min(mapWidth, mapHeight);

	local indoors = GetCVar("minimapZoom")+0 == Minimap:GetZoom() and "outdoor" or "indoor"
	local zoom = Minimap:GetZoom()
	local mapSizeYards = minimap_size[indoors][zoom]

	local yw, yh = MapData:MapArea(GetCurrentMapAreaID(), GetCurrentMapDungeonLevel())
	local pw = yw*mapSizePix/mapSizeYards
	local ph = yh*mapSizePix/mapSizeYards

	if pw==old_pw and ph==oldph then return end
	old_pw, old_ph = pw, ph

	self:SetSize(pw, ph);
end

function Arh_ArchaeologyDigSites_OnLoad(self)
	self:SetFillAlpha(128);
	self:SetFillTexture("Interface\\WorldMap\\UI-ArchaeologyBlob-Inside");
	self:SetBorderTexture("Interface\\WorldMap\\UI-ArchaeologyBlob-Outside");
	self:EnableSmoothing(true);
	--self:SetNumSplinePoints(30);
	self:SetBorderScalar(0.1);
end

function Arh_ArchaeologyDigSites_OnUpdate(self, elapsed)
	self:DrawNone();
	local numEntries = ArchaeologyMapUpdateAll()
	for i = 1, numEntries do
		local blobID = ArcheologyGetVisibleBlobID(i)
		self:DrawBlob(blobID, true)
	end
end

local last_update = 0
function Arh_MainFrame_OnUpdate(frame, elapsed)
	last_update = last_update + elapsed
	if last_update > 0 then
		if Arh_ArchaeologyDigSites_Minimap and Arh_ArchaeologyDigSites_Minimap:IsVisible() then
	                RePositionDigSites_Minimap(Arh_ArchaeologyDigSites_Minimap)
			UpdateDigSitesSize_Minimap(Arh_ArchaeologyDigSites_Minimap)
		end

		last_update = 0
	else
	end
end

local UIParent_Height_old = -1
local MinimapScale_old = -1
ARH_HUDFRAME_SCALE = 1
function UpdateHudFrameSizes()
	local UIParent_Height = UIParent:GetHeight()

	local zoom = Minimap:GetZoom()
	local indoors = GetCVar("minimapZoom")+0 == Minimap:GetZoom() and "outdoor" or "indoor"
	local MinimapScale = minimap_scale[indoors][zoom]

	if UIParent_Height==UIParent_Height_old and MinimapScale==MinimapScale_old then return end
	MinimapScale_old = MinimapScale
	UIParent_Height_old = UIParent_Height


	Arh_HudFrame:SetScale(ARH_HUDFRAME_SCALE)
	local size = UIParent_Height / ARH_HUDFRAME_SCALE
	Arh_HudFrame:SetSize(size, size)

--	local diameter = size * 0.45 * MinimapScale
--	Arh_HudFrame.CompasCircle:SetSize(diameter, diameter)

	local HudPixelsInYard = size / minimap_size[indoors][zoom]

	local melee = 16 * HudPixelsInYard
	Arh_HudFrame.MeleeCircle:SetSize(melee, melee)

--	local radius = size * 0.214 * MinimapScale
--	for k, v in ipairs(Arh_HudFrame.CompasDirections) do
--		v.radius = radius
--	end
end

function Arh_HudFrame_OnLoad()
	originalMinimap = Minimap
	Arh_HudFrame.GetZoom = function(...) return Minimap:GetZoom(...) end
	Arh_HudFrame.SetZoom = function(...) end

	Arh_HudFrame:EnableMouse(false)

	local circle = Arh_HudFrame:CreateTexture()
--	circle:SetTexture([[SPELLS\CIRCLE.BLP]])
--	circle:SetBlendMode("ADD")
--	circle:SetPoint("CENTER")
--	circle:SetVertexColor(0,1,0,0.5)
--	Arh_HudFrame.CompasCircle = circle
--
--	circle = Arh_HudFrame:CreateTexture()
	circle:SetTexture([[SPELLS\CIRCLE.BLP]])
	circle:SetBlendMode("ADD")
	circle:SetPoint("CENTER")
	circle:SetVertexColor(1,0,0,1)
	Arh_HudFrame.MeleeCircle = circle
	circle = Arh_HudFrame:CreateTexture()

--	local directions = {}
--	local indicators = {"N", "NE", "E", "SE", "S", "SW", "W", "NW"}
--	local radius = 10
--	for k, v in ipairs(indicators) do
--		local rot = (0.785398163 * (k-1))
--		local ind = Arh_HudFrame:CreateFontString(nil, nil, "GameFontNormalSmall")
--		local x, y = math.sin(rot), math.cos(rot)
--		ind:SetPoint("CENTER", Arh_HudFrame, "CENTER", x * radius, y * radius)
--		ind:SetText(v)
--		ind:SetShadowOffset(0.2,-0.2)
--		ind:SetTextHeight(20)
--		ind.rad = rot
--		ind.radius = radius
--		ind:SetTextColor(0, 1, 0, 1)
--		tinsert(directions, ind)
--	end
--	Arh_HudFrame.CompasDirections = directions

	Arh_HudFrame:SetFrameStrata("BACKGROUND")
	Arh_HudFrame:SetAlpha(0.7)
end

local last_update_hud = 0
function Arh_HudFrame_OnUpdate(frame, elapsed)
	last_update_hud = last_update_hud + elapsed
	--if last_update_hud > 0.05 then
	if last_update_hud > 0 then
		if not Arh_HudFrame:IsVisible() then return end

		local px, py = GetPlayerMapPosition("player")
		local pa = GetPlayerFacing()
		local japx, japy = Arh_ToYards(px, py);
		UpdateCons(japx, japy, GetPlayerFacing())

		UpdateHudFrameSizes()

--		for k, v in ipairs(Arh_HudFrame.CompasDirections) do
--			local x, y = math.sin(v.rad + pa), math.cos(v.rad + pa)
--			v:ClearAllPoints()
--			v:SetPoint("CENTER", Arh_HudFrame, "CENTER", x * v.radius, y * v.radius)
--		end

		last_update_hud = 0
	end
end

local OriginalRotation
function Arh_HudFrame_OnShow(self)
	OriginalRotation = GetCVar("rotateMinimap")
	if GatherMate2 then
		GatherMate2:GetModule("Display"):ReparentMinimapPins(Arh_HudFrame)
		GatherMate2:GetModule("Display"):ChangedVars(nil, "ROTATE_MINIMAP", "1")
	end
end
function Arh_HudFrame_OnHide(self)
	if GatherMate2 then
		GatherMate2:GetModule("Display"):ReparentMinimapPins(Minimap)
		GatherMate2:GetModule("Display"):ChangedVars(nil, "ROTATE_MINIMAP", OriginalRotation)
	end
end
